import { ComponentPackageItemDSL } from '@lowcode-set-up-platform/dsl';
/**
 * 主要管理组件的版本管理
 */
export interface ComponentVersion {
    name: string;
    libName: string;
    version: string;
    component: JSX.Element;
}
export interface LoadRemoteComponentDependency {
    scriptsList: string[];
    styleList: string[];
    libName: string;
    lifecycle?: Lifecycle;
    content?: () => Record<any, any>;
}
interface Lifecycle {
    onSuccess?: (...com: any) => any;
    onFailure?: (e: Error) => void;
}
interface Options {
    cross: boolean;
}
/**
 * 通过代理模式管理页面所有组价的版本，每一个组件名映射多个版本，渲染组件时指定组件名和版本
 */
export declare class ComponentDependency {
    componentMap: Map<string, Record<string, ComponentVersion>>;
    hostWindow: Window;
    options: Options;
    constructor(hostWindow: Window, options?: Options);
    /**
     * 读取组件
     * @param name
     * @param version
     */
    getComponent(name: string, version: string): Promise<JSX.Element | (() => null) | undefined>;
    /**
     * 设置组价
     * @param name
     * @param version
     * @param libName
     * @param component
     */
    setComponent(name: string, version: string, libName: string, component: JSX.Element): Promise<void>;
    /**
     * 删除组件
     * @param name
     * @param version
     * @param isAll
     */
    deleteComponent(name: string, version: string, isAll?: boolean): boolean | undefined;
    /**
     * 从远程加载资源
     * @param name
     * @param version
     * @param options
     */
    loadRemoteComponent(name: string, version: string, options: LoadRemoteComponentDependency): Promise<{
        scriptDom: HTMLScriptElement[];
        styleDom: HTMLStyleElement[];
        component: JSX.Element;
    }>;
    /**
     * 解析组件
     */
    parseComponentsMap(componentsMap: ComponentPackageItemDSL[]): Promise<Record<string, any>>;
    /**
     * 从远程加载资源
     * @param options
     */
    loadRemoteLib(options: LoadRemoteComponentDependency): Promise<{
        scriptDom: HTMLScriptElement[];
        styleDom: HTMLStyleElement[];
    }>;
    /**
     * 解析依赖包
     */
    parsePackagesMap(componentsMap: ComponentPackageItemDSL[]): Promise<boolean>;
}
export {};
